#!/usr/bin/python
# -*- coding: cp1250 -*-
#
# Skript zajisti spojeni s MSC Mentat, spusteni kodu v MSC Mentat, vypocet a 
# vyhodnoceni ulohy.

#----------- Importovani potrebnych modulu ----------
import os
import py_post

#----------- Definovani funkci ----------------------
def save(soubor,promenna):
# obdoba prikazu save v Matlabu v ASCII formatu
    fid = open(soubor,'w')
    for i in range(len(promenna)):
        fid.write('%f \n' %promenna[i])
    fid.close()
    
def load(soubor):
# obdoba prikazu load v Matlabu
    fid = open(soubor,'r')
    r = fid.readline()
    promenna = []
    while r != '':
        promenna.append(float(r))
        r = fid.readline()
    fid.close()
    return promenna
    
def tisk_spusteni():
# Vytisknuti proc souboru k sestaveni modelu
    fid = open("spust.proc","w")
    fid.write("*py_file_run postav_model.py\n")
    fid.write('*quit yes\n')
    fid.close()
    
def postav_model():
# Spusteni proc souboru k sestaveni modelu
    if os.name == 'nt':
        os.system('mentat -pr spust.proc')          # prikaz win
    elif os.name == 'posix':
        os.system('mentat2008r1 -pr spust.proc')    # prikaz unix
        
def spust_vypocet():
# Vypocet modelu
    if os.name == 'nt':
        os.system('run_marc -j deska')
    elif os.name == 'posix':
        os.system('/usr/sw/marc2008r1/marc2008r1/bin/marc -q background -ml 10000 -j deska')
 
    
def post():
# post-processing
    p = py_post.post_open('deska.t16')
    try:
        p.moveto(1)
    except:
        print 'Chyba pi otvrn souboru deska.t16' 
        return
    
    # porchazeni setu
    for i in range(0,p.sets()):
        s = p.set(i)
        if s.name == 'no_laser':
            i_no_laser = i          # index setu i_no_laser
            break
    
    # prochazeni velicin
    for i_scalar in range(0,p.node_scalars()):
        if p.node_scalar_label(i_scalar) == 'Displacement Z': 
            i_dislpacement = i_scalar               # index Displacement Z
            break
    
    # prochazeni kontaktnich teles
    for i_cbody in range(0,p.cbodies()):
        if p.cbody_name(i_cbody) == 'cbo_raznik':      
            i_cbo_raznik = i_cbody                  # index telesa cbo_raznik
            break
    
    sila = []
    pruhyb = []
    cas = []
    
    for i in range(1,p.increments()): 
        
        print 'Prochazim '+str(i)+ '. inkrement' 
        
        p.moveto(i)
        cas.append(p.time)
        
        pom = p.cbody_force(i_cbo_raznik)
        sila.append(pom[2])
        
        pruhyb.append(p.node_scalar(i_no_laser-1, i_dislpacement))
            
    p.close()
    
#    print 'Sila: %s' %sila
#    print 'Pruhyb %s:' %pruhyb    

    save('sila.txt',sila)
    save('pruhyb.txt',pruhyb)
    
def grafy():
# fuknce pro vytisteni grafu vypocteneho pruhybu a kontaktni sily
    try:
        import matplotlib.pyplot as plt
    except:
        print "Neni nainstalovan modul matplotlib.pyplot nelze sestrijit grafy!!"
        return
    
    sila = load('sila.txt')
    pruhyb = load('pruhyb.txt')
    
    plt.subplot(1,2,1)    
    plt.plot(sila)
    plt.title("Kontaktni sila")
    
    plt.subplot(1,2,2)
    plt.plot(pruhyb)
    plt.title("Pruhyb laminatu")
     
    plt.show()
    

def main():
# Funkce kodu vykonaneho po spusteni
    print '--- Tisk proc souboru pro spojeni s pythonem ---\n'
    
    tisk_spusteni();
    
    print '--- Spousteni Mentatu a stavba modelu ---\n'
    
    postav_model()
        
    print('--- Spousteni vypocetu! ---\n')
    
    spust_vypocet()
           
    print('--- Spousti se postprocessing! ---\n')
    
    post()
    grafy()
    
# ---------------- samotny kod programu ------------------------
if __name__ == '__main__':
    main()