#Skript spusteny z MSC Mentat pro stavbu modelu pomoci prikazu MSC Mentat

#----------- Importovani potrebnych modulu ----------
from py_mentat import *
import deska

def plocha(roh1,roh2,roh3,roh4):
# Generuje plochu s definovanymi rohovymi body
    
    py_send('*set_surface_type quad\n')
    py_send('*add_surface \n')    
    py_send('point('+str(roh1)[1:-1]+')')
    py_send('point('+str(roh2)[1:-1]+')')
    py_send('point('+str(roh3)[1:-1]+')')
    py_send('point('+str(roh4)[1:-1]+')')
   
def vzdalenost_bodu(bod1,bod2):
# vypocte vzdalenost mezi dvema body [x,y,z]
    return ((bod1[0]-bod2[0])**2+(bod1[1]-bod2[1])**2+(bod1[2]-bod2[2])**2)**0.5
    
def mesh_deska(roh1,roh2,roh3,roh4,element):
# Generuje prvidelny mesh desky s definovanymi rohy a velikosti elemntu
    
    plocha(roh1,roh2,roh3,roh4)
    vzdalenost = vzdalenost_bodu(roh1,roh2)
    pocet = max(int(vzdalenost/element),1)
    vzdalenost = vzdalenost_bodu(roh2,roh3)
    pocet = [pocet,max(int(vzdalenost/element),1)]
    
    py_send('*set_convert_divisions '+str(pocet[0])+' '+str(pocet[1]))
    py_send('*set_convert_bias_factors 0 0')
    
    py_send('*convert_surfaces max_surface_id() #')
    
    
def usecka(body , element, smer):
# Vytiskne usecku s rozmerem "rozmer" ve smeru dane cislem "smer" a rozdeli
# ji vyhledem k velikosti elementu "element"

    deleni = []
    body.sort()
    
    for i_body in range(len(body)-1):
        rozmer = body[i_body+1]-body[i_body]
        n_deleni = max(1,int(rozmer/element))
        
        for i in range(0,n_deleni+1):
            deleni.append(body[i_body]+i*rozmer/n_deleni)
    
    deleni = list(set(deleni))
    deleni.sort()
    
    py_send('*set_curve_type polyline\n')
    py_send('*add_curves\n')
    
    for i in range(len(deleni)):
        if smer == 1:
            py_send('point(%f, 0, 0)\n' % deleni[i])
        elif smer == 2:
            py_send('point(0, %f, 0)\n' % deleni[i])
        elif smer == 3:
            py_send('point(0, 0, %f)\n' % deleni[i])
    py_send('#\n')

    if echo:
        print("Vykeslena usecka s body: "+str(deleni)+"\n")
    return deleni 

def modeluj_raznik(deska):
# vytvori model razniku
    py_send('*select_clear\n')
    py_send('*visible_selected\n')

    py_send('*set_curve_type line\n')
    py_send('*add_curves\n')
    py_send('point(7.5,0,0)\n')
    py_send('point(0,0,0)\n')
    py_send('point(0,0,0)\n')
    py_send('point(0,7.5,0)\n')
    
    py_send('*set_curve_type arc_cpa\n')
    py_send('*add_curves\n')
    py_send('0 0 0\n')
    py_send('7.5 0 0\n')
    py_send(str(90)+'\n')
    
    py_send('*set_curve_div_type_fix_avgl *set_curve_div_avgl\n')
    py_send('1\n')
    py_send('*apply_curve_divisions\n')
    py_send('max_curve_id()\n')
    py_send('max_curve_id()-1\n')
    py_send('max_curve_id()-2\n')
    py_send('#\n')
    
    py_send('*af_planar_quadmesh\n')
    py_send('max_curve_id()\n')
    py_send('max_curve_id()-1\n')
    py_send('max_curve_id()-2\n')
    py_send('#\n')
    
    py_send('*store_nodes no_raznik all_visible\n')
    py_send('*store_elements el_raznik all_visible\n')
    
    py_send('*detach_elements el_raznik\n')
     
    # modelovani ctvrtiny dolni plochy razniku
    py_send('*set_curve_type arc_ppp\n')
    py_send('*add_curves\n')
    py_send('-7.5 '+str(+deska.rozmery[2]/2 + 2.28)+' 0\n')
    py_send('7.5 '+str(+deska.rozmery[2]/2 + 2.28)+' 0\n')
    py_send('0 '+str(+deska.rozmery[2]/2)+' 0\n')
    
    py_send('*sub_divisions 2 2 2 \n')
    py_send('*subdivide_curves_real \n')
    py_send('max_curve_id() #\n')
    py_send('*remove_curves\n')
    py_send('max_curve_id()-1 #\n')
    
    py_send('*revolve_angles 0 90\n')
    py_send('*revolve_curves max_curve_id() #\n')
    
    py_send('*move_reset\n')
    py_send('*set_move_rotation x 90\n')
    py_send('*move_surfaces max_surface_id() #\n')
    
    py_send('*move_to_geom_mode_directed\n')
    py_send('*set_move_to_geom_direction 0 0 1\n')
    py_send('*move_nodes_to_surface max_surface_id() no_raznik\n')
    
    py_send('*expand_reset\n')
    py_send('*set_expand_translation z 1\n')
    py_send('*set_expand_repetitions 1\n')
    py_send('*expand_elements el_raznik\n')
    py_send('*expand_reset\n')
    
    py_send('*select_clear\n')
    py_send('*select_sets no_raznik\n')
    py_send('*select_mode_intersect\n')
    py_send('*select_method_point_dist\n')
    py_send('*set_select_distance 0.1\n')
    py_send('*select_nodes 0 0 '+str(deska.rozmery[2]/2)+'\n')
    py_send('*store_nodes no_sila all_selected\n')
    py_send('*select_reset\n')

    py_send('*duplicate_reset\n')
    py_send('*set_duplicate_rotation z 90\n')
    py_send('*set_duplicate_repetitions 4\n')
    py_send('*duplicate_elements el_raznik\n')
    
    py_send('*remove_unused_nodes\n')
    py_send('*sweep_nodes no_raznik\n')
    
    py_send('*move_reset\n')
    py_send('*set_move_translation x '+str(deska.rozmery[0]/2)+'\n')
    py_send('*set_move_translation y '+str(deska.rozmery[1]/2)+'\n')
    py_send('*move_elements el_raznik #\n')
    
    # --- Geometry ---
    py_send('*new_geometry\n')
    py_send('*geometry_name geo_raznik')
    py_send('*geometry_type mech_three_solid\n')
    py_send('*add_geometry_elements el_raznik\n')
    
    # --- Material ---
    py_send('*select_clear\n')
    py_send('*select_set el_raznik')
    objem  = py_get_float('element_volume(0)')
    
    if echo:
        print("Objem prvku razniku je "+str(objem))
        
    py_send('*select_clear\n')
    
    py_send('*new_mater standard *mater_option general:state:solid\n')
    py_send('*mater_name mat_raznik\n')
    py_send('*mater_param general:mass_density '+str(1.926/objem)+'\n')
    py_send('*mater_param structural:youngs_modulus 210\n')
    py_send('*mater_param structural:poissons_ratio 0.3\n')
    py_send('*add_mater_elements el_raznik\n')
    
    # --- Pocatecni pominky ---
    py_send('*new_icond\n')
    py_send('*icond_name ico_raznik\n')
    py_send('*icond_type velocity\n')
    py_send('*icond_dof z *icond_dof_value z -3.5\n')
    py_send('*add_icond_nodes\n')
    py_send('no_raznik\n')
    
    # --- Okrajove podminky ---
    py_send('*new_apply')
    py_send('*apply_name bco_raznik')
    py_send('*apply_type fixed_displacement\n')
    py_send('*apply_dof x *apply_dof_value x\n')
    py_send('*apply_dof y *apply_dof_value y\n')
    py_send('*apply_dof rx *apply_dof_value rx\n')
    py_send('*apply_dof ry *apply_dof_value ry\n')
    py_send('*apply_dof rz *apply_dof_value rz\n')
    
    py_send('*select_clear\n')
    py_send('*select_method_point_dist\n')
    py_send('*set_select_distance 0.1\n')
    py_send('*select_nodes '+str(deska.rozmery[0]/2)+','+str(deska.rozmery[1]/2)+','+str(+deska.rozmery[2]/2)+'\n')
    py_send('*select_nodes '+str(deska.rozmery[0]/2)+','+str(deska.rozmery[1]/2)+','+str(+deska.rozmery[2]/2+1)+'\n')
  
    py_send('*add_apply_nodes all_selected #\n')
    py_send('*select_reset\n')
    
    if echo:
        print('Vytvoren model razniku! \n')

def modeluj_podpery(deska):
# vytvori model podper
    py_send('*select_clear\n')
    py_send('*visible_selected\n')
    
    #plocha([-21,-10,-0.75],[21,-10,-0.75],[21,100,-0.75],[-21,100,-0.75])
    #leva_podpera_id = py_get_int('max_surface_id()')
    
    #plocha([90-21,-10,-0.75],[90+21,-10,-0.75],[90+21,100,-0.75],[90-21,100,-0.75])
    #prava_podpera_id = py_get_int('max_surface_id()')
    
    mesh_deska([-21,-10,-0.75],[21,-10,-0.75],[21,100,-0.75],[-21,100,-0.75],50)
    mesh_deska([90-21,-10,-0.75],[90+21,-10,-0.75],[90+21,100,-0.75],[90-21,100,-0.75],50)
    
    py_send('*store_elements el_podpery all_visible\n')
    py_send('*store_nodes no_podpery all_visible\n')
    
    py_send('*expand_reset\n')
    py_send('*set_expand_translation z -5\n')
    py_send('*set_expand_repetitions 1\n')
    py_send('*expand_elements el_podpery\n')
    py_send('*expand_reset\n')
    
    py_send('*sweep_nodes no_podpery\n')
       
    # --- Geometry ---
    py_send('*new_geometry\n')
    py_send('*geometry_name geo_podpery')
    py_send('*geometry_type mech_three_solid\n')
    py_send('*add_geometry_elements el_podpery\n')
    
    # --- Material ---
    py_send('*new_mater standard *mater_option general:state:solid\n')
    py_send('*mater_name mat_podpery\n')
    py_send('*mater_param general:mass_density 7800e-9\n')
    py_send('*mater_param structural:youngs_modulus 210\n')
    py_send('*mater_param structural:poissons_ratio 0.3\n')
    py_send('*add_mater_elements el_podpery\n')
    
    # --- Okrajove podminky ---
    py_send('*new_apply')
    py_send('*apply_name bco_vetknuti_podper')
    py_send('*apply_type fixed_displacement\n')
    py_send('*apply_dof x *apply_dof_value x\n')
    py_send('*apply_dof y *apply_dof_value y\n')
    py_send('*apply_dof z *apply_dof_value y\n')
    py_send('*apply_dof rx *apply_dof_value rx\n')
    py_send('*apply_dof ry *apply_dof_value ry\n')
    py_send('*apply_dof rz *apply_dof_value rz\n')
    py_send('*add_apply_nodes no_podpery #\n')
    
    #return [leva_podpera_id,prava_podpera_id]

def modeluj_desku(deska):
# prida impaktovanou desku do modelu
    
    # --- mesh ---
    py_send('*select_clear\n')
    py_send('*visible_selected\n')
    
    deleni_x = [0,deska.laser[0],deska.rozmery[0]]
    deleni_y = [0,deska.laser[1],deska.rozmery[1]]

    deleni_x = usecka(deleni_x, deska.element, 1)
    deleni_y = usecka(deleni_y, deska.element, 2)

    py_send('*set_surface_type driven\n')
    py_send('*add_surface max_curve_id()-1 max_curve_id()\n')
    
    py_send('*set_convert_divisions '+str(len(deleni_x)-1)+' '+str(len(deleni_y)-1)+'\n')
    py_send('*convert_surfaces max_surface_id() #\n')

    py_send('*store_elements el_deska \n')
    py_send('all_visible\n')
    
    # vybrani uzlu laseru
    py_send('*select_clear\n')
    py_send('*select_nodes all_visible\n')
    py_send('*select_mode_intersect\n')
    py_send('*select_method_point_dist\n')
    py_send('*set_select_distance %f \n' % (deska.element/4))
    py_send('*select_nodes %f %f 0 \n' %(deska.laser[0], deska.laser[1]))
    py_send('*store_nodes no_laser all_selected\n')
    py_send('*select_reset\n')
    
    # --- Geometry ---
    py_send('*new_geometry\n')
    py_send('*geometry_name geo_deska')
    py_send('*geometry_type mech_three_shell\n')
    py_send('*geometry_param thick 1.5\n')
    py_send('*add_geometry_elements el_deska\n')
    
    # --- Material ---    
    py_send('*new_mater standard *mater_option general:state:solid\n')
    py_send('*mater_name mat_vrstva\n')
    py_send('*mater_param general:mass_density '+str(1510e-9)+'\n')
    py_send('*mater_option structural:type:elast_plast_ortho\n')
    py_send('*mater_param structural:youngs_modulus1 153.4\n')
    py_send('*mater_param structural:youngs_modulus2 7.8\n')
    py_send('*mater_param structural:youngs_modulus3 7.8\n')
    py_send('*mater_param structural:poissons_ratio12 0.28\n')
    py_send('*mater_param structural:poissons_ratio23 0.4\n')
    py_send('*mater_param structural:poissons_ratio31 0.01\n')
    py_send('*mater_param structural:shear_modulus12 4.5\n')
    py_send('*mater_param structural:shear_modulus23 4\n')
    py_send('*mater_param structural:shear_modulus31 4.5\n')
    
    py_send('*new_mater composite *mater_option general:state:solid\n')
    py_send('*mater_name mat_deska\n')
    py_send('*mater_append_submat mat_vrstva\n')
    py_send('*mater_submat_param 1 general:compos_ply_thick_percent 25\n')
    py_send('*mater_submat_param 1 general:compos_ply_orient_angle_deg 0\n')
    
    py_send('*mater_append_submat mat_vrstva\n')
    py_send('*mater_submat_param 2 general:compos_ply_thick_percent 25\n')
    py_send('*mater_submat_param 2 general:compos_ply_orient_angle_deg 90\n')
    
    py_send('*mater_append_submat mat_vrstva\n')
    py_send('*mater_submat_param 3 general:compos_ply_thick_percent 25\n')
    py_send('*mater_submat_param 3 general:compos_ply_orient_angle_deg 90\n')
    
    py_send('*mater_append_submat mat_vrstva\n')
    py_send('*mater_submat_param 4 general:compos_ply_thick_percent 25\n')
    py_send('*mater_submat_param 4 general:compos_ply_orient_angle_deg 0\n')
    
    py_send('*add_mater_elements el_deska\n')
    
    # --- Orientace ---
    py_send('*new_orient\n')
    py_send('*orient_name ori_deska\n')
    py_send('*add_mater_elements el_deska\n')
    py_send('*orient_type edge12\n')
    py_send('*add_orient_elements el_deska\n')
    
    
def pridej_kontakt(deska):
# prida do modelu kontatk mezi deskou, raznikem a podperama
    
    # ---- Contact body ---- 
    py_send('*new_contact_body\n')
    py_send('*contact_body_name cbo_raznik\n')
    py_send('*contact_deformable\n')
    py_send('*contact_value friction 0.2\n')
    py_send('*add_contact_body_elements el_raznik\n')
    
    py_send('*new_contact_body\n')
    py_send('*contact_body_name cbo_deska\n')
    py_send('*contact_deformable\n')
    py_send('*contact_value friction 0.2\n')
    py_send('*add_contact_body_elements el_deska\n')
    
    py_send('*new_contact_body\n')
    py_send('*contact_body_name cbo_podpery\n')
    py_send('*contact_deformable\n')
    py_send('*contact_value friction 0.2\n')
    py_send('*add_contact_body_elements el_podpery #\n')    
    
    # --- Contact Table ---
    py_send('*new_contact_table\n')
    py_send('*contact_table_name cta_vse\n')
    py_send('*contact_table_entry 1 2')
    py_send('*contact_table_option $ctbody1 $ctbody2 contact_type:touching\n')
    py_send('*contact_table_entry 1 3')
    py_send('*contact_table_option $ctbody1 $ctbody2 contact_type:touching\n')
    py_send('*contact_table_entry 2 3')
    py_send('*contact_table_option $ctbody1 $ctbody2 contact_type:touching\n')
    
def pridej_loadcase():
# prida do modelu loadcase
    
    py_send('*new_loadcase *loadcase_type struc:dyn_trans')
    py_send('*loadcase_name loa_transientni')
    py_send('*loadcase_value time 4')
    py_send('*loadcase_value nsteps 40')
    py_send('*loadcase_ctable cta_vse')
    py_send('*loadcase_value maxrec 99')
    py_send('*loadcase_value force 0.05')
    
def pridej_job():
# prida do modelu job
    
    py_send('*new_job *job_class structural')
    py_send('*job_name job_deska')
    py_send('*job_option dimen:three')
    py_send('*job_option strain:large')
    py_send('*job_option tshear:on')
    py_send('*add_job_loadcase loa_transientni')
    
    py_send('*element_type 7 el_raznik')
    py_send('*element_type 7 el_podpery')
    py_send('*element_type 75 el_deska')

def zacisteni():
# docisteni modelu
    py_send('*clear_geometry')
    
    py_send('*select_clear\n')
    py_send('*invisible_selected\n')
    py_send('*fill_view\n')     
       
def main():
# hlavni trida vytvori instaci tridy deska a vytvori model teto instance
    global echo
    echo = 1        # vypisy behem programu
    
    material = deska.Material()     # definovani materialu
    vzorek = deska.Deska(material)  # definovani desky
    
    py_send('*show_view 2\n')
    py_send('*elements_solid *regenerate')
    
    modeluj_desku(vzorek)
    modeluj_raznik(vzorek)
    modeluj_podpery(vzorek)
    
    zacisteni()
    
    pridej_kontakt(vzorek)
    pridej_loadcase()
    pridej_job()
    
    py_send('*renumber_all\n')    # zivotne dulezite pro postpocessing
    
    py_send('*write_marc deska.dat yes\n')
    py_send('*save_as_model deska.mud yes\n')  

if __name__ == '__main__':
    py_connect("127.0.0.1",40007)
    main()
    py_disconnect()